# -*- coding: utf-8 -*-
"""

This is the gui where the spinboxes and checkboxes for controlling the mc daq live. Nevertheless, measurements are not
controlled from this Gui. It is more for searching and exploring the paremeters. 

@author: Usuario
"""
import sys
from PyQt5 import QtGui, QtCore
#import qdarkstyle
import os
from PyQt5.QtWidgets import QLabel

os.environ['PYQTGRAPH_QT_LIB'] = 'PyQt5'

from subclasses.controllayouts import LaserControlLayout
from subclasses.controllayouts import LaserScanLayout
from resources.myToolbar import myToolbarMenu
from pyqtgraph import dockarea
from MCDAQcontrolsignals import anal_control_signal
from MCDAQcontrolsignals import digital_control_signal

class ControlGui(QtGui.QFrame):

    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        self.menuBar = QtGui.QMenuBar(self)
        self.area = dockarea.DockArea()
        self.grid = QtGui.QGridLayout()
        self.grid.addWidget(QLabel(' '), 0, 1)
        self.grid.addWidget(self.area, 1, 0)
        self.grid.setSpacing(0)
        self.setLayout(self.grid)

        self.dock_397 = dockarea.Dock('397', (100, 100))
        self.dock_866 = dockarea.Dock('866', (100, 100))
        self.dock_423 = dockarea.Dock('423', (100, 100))
        self.dock_397_scan = dockarea.Dock('397 scan', (100, 100))
        self.dock_866_scan = dockarea.Dock('866 scan', (100, 100))
        self.dock_423_scan = dockarea.Dock('423 scan', (100, 100))

        self.dock_electrodes = dockarea.Dock('dcElectrodes', (100, 100))
        
        self.area.addDock(self.dock_electrodes)
        self.area.addDock(self.dock_397, 'bottom', self.dock_electrodes)
        self.area.addDock(self.dock_866, 'bottom', self.dock_397)
        self.area.addDock(self.dock_423, 'bottom', self.dock_866)        
        self.area.addDock(self.dock_397_scan, 'right', self.dock_397)
        self.area.addDock(self.dock_866_scan, 'right', self.dock_866)
        self.area.addDock(self.dock_423_scan, 'right', self.dock_423)



        self.laser397_control_layout = LaserControlLayout('397', checkboxes=['shutter397'], spinboxes = ['piezoA397', 'piezoB397'])
        self.laser397_scan_layout = LaserScanLayout(name='piezoB397', signal_to_scan_layout=self.laser397_control_layout)
        self.dock_397.addWidget(self.laser397_control_layout, 0, 0)
        self.dock_397_scan.addWidget(self.laser397_scan_layout, 0, 1)
        
        self.laser866_control_layout = LaserControlLayout('866', checkboxes=['shutter866', 'shutter866WM'], spinboxes = ['piezoA866', 'piezoB866'], blinkCheckbox = ['shutter866'])
        self.laser866_scan_layout = LaserScanLayout(name='piezoB866', signal_to_scan_layout=self.laser866_control_layout)
        self.dock_866.addWidget(self.laser866_control_layout)
        self.dock_866_scan.addWidget(self.laser866_scan_layout)

        self.laser423_control_layout = LaserControlLayout('423', checkboxes=['shutter423'], spinboxes = ['piezo423'])
        self.laser423_scan_layout = LaserScanLayout(name='piezo423', signal_to_scan_layout=self.laser423_control_layout)
        self.dock_423.addWidget(self.laser423_control_layout)
        self.dock_423_scan.addWidget(self.laser423_scan_layout)

        self.electrodes_control_layout = LaserControlLayout('electrodes', checkboxes=[], spinboxes = ['dcA', 'dcB', 'compC', 'compD', 'compOven', 'compExYb'], custom_geometry=True)
        self.dock_electrodes.addWidget(self.electrodes_control_layout)

        self.electrodes_control_layout.addWidget(QtGui.QLabel('dcA'), 0, 0)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['dcA'], 0, 1)
        self.electrodes_control_layout.addWidget(QtGui.QLabel('dcB'), 1, 0)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['dcB'], 1, 1)
        self.electrodes_control_layout.addWidget(QtGui.QLabel('compC'), 0, 2)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['compC'], 0, 3)
        self.electrodes_control_layout.addWidget(QtGui.QLabel('compD'), 1, 2)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['compD'], 1, 3)
        self.electrodes_control_layout.addWidget(QtGui.QLabel('compOven'), 0, 4)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['compOven'], 0, 5)
        self.electrodes_control_layout.addWidget(QtGui.QLabel('compExYb'), 1, 4)
        self.electrodes_control_layout.addWidget(self.electrodes_control_layout.spinboxes['compExYb'], 1, 5)

        self.toolbar = myToolbarMenu(self)
        self.autosave_timer = QtCore.QTimer()
        self.autosave_timer.timeout.connect(lambda: self.toolbar.save('config.ini'))
        self.autosave_timer.start(10000)
        
        self.show()

    def make_connections(self, backend):
        # external connections
        scan_layout_list = [scan_layout for scan_layout in LaserScanLayout._registry if scan_layout.name == backend.name]
        for scan_layout in scan_layout_list:
            scan_layout.scanActionSignal.connect(backend.scan_action)
            backend.scanStepSignal.connect(scan_layout.scan_step)
            backend.scanEndSignal.connect(scan_layout.scan_end)


#%%
if __name__ == '__main__':
    app = QtGui.QApplication([])
    controlGui = ControlGui()
    piezoA397 = anal_control_signal('piezoA397', channel=0, ao_type='cavity_piezo', out_minimum = 0, out_maximum = 4)
    piezoB397 = anal_control_signal('piezoB397', channel=1, ao_type='cavity_piezo')
    piezoA866 = anal_control_signal('piezoA866', channel=2, ao_type='cavity_piezo', out_minimum = 0, out_maximum = 4)
    piezoB866 = anal_control_signal('piezoB866', channel=3, ao_type='cavity_piezo')
    piezo423 = anal_control_signal('piezo423', channel=4, ao_type='laser_piezo')
    
    trapDCA = anal_control_signal('dcA', channel = 5, ao_type='electrode')
    trapDCB = anal_control_signal('dcB', channel = 6, ao_type='electrode')
    trapCOMPC = anal_control_signal('compC', channel = 7, ao_type='electrode')
    trapCOMPD = anal_control_signal('compD', channel = 8, ao_type='electrode')
    trapOven = anal_control_signal('compOven', channel = 9, ao_type='electrode')
    trapExYb = anal_control_signal('compExYb', channel = 10, ao_type='electrode')


    shutter866 = digital_control_signal('shutter866', channel = 0)
    shutter397 = digital_control_signal('shutter397', channel = 1)
    shutter423 = digital_control_signal('shutter423', channel = 2)
    shutter866WM = digital_control_signal('shutter866WM', channel = 3)

    mcDAQthread = QtCore.QThread()
    for ao in anal_control_signal._registry:
        controlGui.make_connections(ao)
        ao.make_connections(controlGui)
        ao.moveToThread(mcDAQthread)
        ao.scanTimer.moveToThread(mcDAQthread)
        ao.scanTimer.timeout.connect(ao.scan_event)
    for do in digital_control_signal._registry:
        do.make_connections(controlGui)
        do.moveToThread(mcDAQthread)

    mcDAQthread.start()
    controlGui.toolbar.load()
    
    sys.exit(app.exec_())