#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
tip: pyuic5 -x ventana.ui -o ventana_ui.py
@author: martindrech

The ui skeleton is built with qtdesigner and its structur lies in the measurement_ui.py file.

"""

from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import QSettings
from PyQt5.QtWidgets import QAction, QFileDialog, QInputDialog, QWidget
from measurement_ui import Ui_Frame
from MCDAQcontrolsignals import anal_control_signal

class MeasurementFrame(QtWidgets.QFrame, Ui_Frame):
    
    
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        for ao in anal_control_signal._registry:
            self.selectScanSignal.addItem(ao.name)
        try:
            self.configure_spinBoxes(anal_control_signal._registry[0].name)
        except IndexError:
            print('No analog control signals defined. ')

        self.treeModel = QtGui.QStandardItemModel()
        self.treeView.setAlternatingRowColors(True)
        self.treeView.setSortingEnabled(True)
        self.treeView.setHeaderHidden(False)
        self.treeView.setSelectionBehavior(QtGui.QAbstractItemView.SelectItems)
        self.treeView.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.treeModel.setHorizontalHeaderLabels(['Parameter', 'Value'])
        self.treeView.setModel(self.treeModel)


        self.treeModel.insertRow(0)            
        self.treeModel.setData(self.treeModel.index(0, 0), r'dV')
        self.treeModel.setData(self.treeModel.index(0, 1), '?')
    
        self.saveMeasureButton.setEnabled(False)


    def select_folder(self):
        directory = QFileDialog.getExistingDirectory(self, 'Choose directory')
        self.folderBrowser.setText(directory)

    @QtCore.pyqtSlot(str)
    def configure_spinBoxes(self, ao_name):
        ao = self.get_selected_ao()
        self.startValue.setMinimum(ao.min)
        self.startValue.setMaximum(ao.max)
        self.startValue.setSingleStep(0.1)

        self.endValue.setMinimum(ao.min)
        self.endValue.setMaximum(ao.max)
        self.endValue.setSingleStep(0.1)
        
        self.stepsNum.setMaximum(1000000)
        
    def get_selected_ao(self):
        try:
            return [ao for ao in anal_control_signal._registry if ao.name == self.selectScanSignal.currentText()][0]
        except IndexError:
            print('No analog control signals defined. ')


    def update_tree(self):
        try:
            self.treeModel.setData(self.treeModel.index(0, 1), (self.endValue.value()-self.startValue.value())/self.stepsNum.value() )
        except:
            self.treeModel.setData(self.treeModel.index(0, 1), '?')

if __name__ == '__main__':
    import sys
    from PyQt5 import QtWidgets
    from measurement_ui import Ui_Frame
    piezoA397 = anal_control_signal('piezoA397', channel=0, ao_type='cavity_piezo', out_minimum = 0, out_maximum = 4)
    piezoB397 = anal_control_signal('piezoB397', channel=1, ao_type='cavity_piezo')
    piezoA866 = anal_control_signal('piezoA866', channel=2, ao_type='cavity_piezo', out_minimum = 0, out_maximum = 4)
    piezoB866 = anal_control_signal('piezoB866', channel=3, ao_type='cavity_piezo')
    piezo423 = anal_control_signal('piezo423', channel=4, ao_type='laser_piezo')    
    trapDCA = anal_control_signal('dcA', channel = 5, ao_type='electrode')
    trapDCB = anal_control_signal('dcB', channel = 6, ao_type='electrode')
    trapCOMPC = anal_control_signal('compC', channel = 7, ao_type='electrode')
    trapCOMPD = anal_control_signal('compD', channel = 8, ao_type='electrode')
    trapOven = anal_control_signal('compOven', channel = 9, ao_type='electrode')
    trapExYb = anal_control_signal('compExYb', channel = 10, ao_type='electrode')

    app = QtWidgets.QApplication(sys.argv)
    gui = MeasurementFrame()    
    gui.show()
    sys.exit(app.exec_())




